/**
 * Copyright (c) 2008-2009 The Open Source Geospatial Foundation
 * 
 * Published under the BSD license.
 * See http://svn.geoext.org/core/trunk/geoext/license.txt for the full text
 * of the license.
 */

/** api: example[feature-grid]
 *  Grid with Features
 *  ------------------
 *  Synchronize selection of features between a grid and a layer.
 */

var mapPanel, store, gridPanel, mainPanel;

Ext.onReady(function() {
    // create map instance
    var map = new OpenLayers.Map();
    var wmsLayer = new OpenLayers.Layer.WMS(
        "vmap0",
        "http://labs.metacarta.com/wms/vmap0",
        {layers: 'basic'}
    );

    // create vector layer
    var vecLayer = new OpenLayers.Layer.Vector("vector");
    map.addLayers([wmsLayer, vecLayer]);

    var ctrl, toolbarItems = [], action, actions = {};

    // ZoomToMaxExtent control, a "button" control
    action = new GeoExt.Action({
        control: new OpenLayers.Control.ZoomToMaxExtent(),
        map: map,
        text: "max extent",
        tooltip: "zoom to max extent"
    });
    actions["max_extent"] = action;
    toolbarItems.push(action);
    toolbarItems.push("-");

    // Navigation control and DrawFeature controls
    // in the same toggle group
    action = new GeoExt.Action({
        text: "nav",
        control: new OpenLayers.Control.Navigation(),
        map: map,
        // button options
        toggleGroup: "draw",
        allowDepress: false,
        pressed: true,
        tooltip: "navigate",
        // check item options
        group: "draw",
        checked: true
    });
    actions["nav"] = action;
    toolbarItems.push(action);

    action = new GeoExt.Action({
        text: "draw point",
        control: new OpenLayers.Control.DrawFeature(
            vecLayer, OpenLayers.Handler.Point,
            {
                eventListeners: {
                    featureadded: function(evt) {
                        evt.feature.attributes.longitude = evt.feature.geometry.x;
                        evt.feature.attributes.latitude = evt.feature.geometry.y;
                        evt.feature.layer.events.triggerEvent("featuremodified", {feature: evt.feature});
                     }
                }
            }
        ),
        map: map,
        // button options
        toggleGroup: "draw",
        allowDepress: false,
        tooltip: "draw point",
        // check item options
        group: "draw"
    });
    actions["draw_point"] = action;
    toolbarItems.push(action);

    toolbarItems.push("-");

    var selFeatCtlCfg = {
        type: OpenLayers.Control.TYPE_TOGGLE,
        hover: true,
        eventListeners: {
            featurehighlighted : function(evt) {
                var feat = evt.feature;
                var map = feat.layer.map;
                var coord = new OpenLayers.LonLat(feat.geometry.x, feat.geometry.y);
                if (!map.getExtent().containsLonLat(coord))
                    map.panTo(coord);
            }
        }
    } ;

    var selFeatCtl = new OpenLayers.Control.SelectFeature(vecLayer, selFeatCtlCfg);
//    var selFeatCtl2 = new OpenLayers.Control.SelectFeature(vecLayer, selFeatCtlCfg);

    // SelectFeature control, a "toggle" control
    action = new GeoExt.Action({
        text: "select",
        control: selFeatCtl,
        map: map,
        // button options
        enableToggle: true,
        tooltip: "select feature"
    });
    actions["select"] = action;
    toolbarItems.push(action);
    toolbarItems.push("-");

    // Navigation history - two "button" controls
    ctrl = new OpenLayers.Control.NavigationHistory();
    map.addControl(ctrl);

    action = new GeoExt.Action({
        text: "previous",
        control: ctrl.previous,
        disabled: true,
        tooltip: "previous in history"
    });
    actions["previous"] = action;
    toolbarItems.push(action);

    action = new GeoExt.Action({
        text: "next",
        control: ctrl.next,
        disabled: true,
        tooltip: "next in history"
    });
    actions["next"] = action;
    toolbarItems.push(action);
    toolbarItems.push("->");

    // Reuse the GeoExt.Action objects created above
    // as menu items
    toolbarItems.push({
        text: "menu",
        menu: new Ext.menu.Menu({
            items: [
                // ZoomToMaxExtent
                actions["max_extent"],
                // Nav
                new Ext.menu.CheckItem(actions["nav"]),
                // Draw point
                new Ext.menu.CheckItem(actions["draw_point"]),
                // Select control
                new Ext.menu.CheckItem(actions["select"]),
                // Navigation history control
                actions["previous"],
                actions["next"]
            ]
        })
    });

    // create map panel
    mapPanel = new GeoExt.MapPanel({
        title: "Map",
        region: "center",
        height: 400,
        width: 600,
        map: map,
        center: new OpenLayers.LonLat(5, 45),
        zoom: 6,
        tbar: toolbarItems
    });
 
    // create feature store, binding it to the vector layer
    store = new GeoExt.data.FeatureStore({
        layer: vecLayer,
        fields: [
            {
                name: 'id',
                type: 'int'
            },
            {
                name: 'latitude',
                type: 'float'
            },
            {
                name: 'longitude',
                type: 'float'
            }
        ],
        proxy: new GeoExt.data.ProtocolProxy({
            protocol: new OpenLayers.Protocol.HTTP({
                url: "data/points.json",
                format: new OpenLayers.Format.GeoJSON()
            })
        }),
        autoLoad: true
    });

    var selModel = new GeoExt.grid.FeatureSelectionModel({selectControl : selFeatCtl});

    // create grid panel configured with feature store
    gridPanel = new Ext.grid.GridPanel({
        title: "Feature Grid",
        region: "east",
        store: store,
        width: 500,
        columns: [
            {
                header: "ID",
                width: 50,
                dataIndex: "id"
            },
            {
                header: "Latitude",
                width: 200,
                dataIndex: "latitude"
            },
            {
                header: "Longitude",
                width: 200,
                dataIndex: "longitude"
            }
        ],
//        sm: new GeoExt.grid.FeatureSelectionModel()
        sm: selModel
    });

    // create a panel and add the map panel and grid panel
    // inside it
    mainPanel = new Ext.Panel({
        renderTo: "mainpanel",
        layout: "border",
        height: 400,
        width: 920,
        items: [mapPanel, gridPanel]
    });
});

